@@ -15,7 +15,8 @@ from flask import (
     url_for
 )
 from flask_cors import cross_origin
-from werkzeug import MultiDict
+from requests.exceptions import RequestException
+from werkzeug.datastructures import MultiDict
 from wtforms import validators
 
 from .. import (
@@ -60,12 +61,15 @@ def get_docs_link():
 def get_notice(r):
     notice = None
     if not r.cookies.get('dismiss_notice'):
-        response = requests.get(
-            app.config['TILEDRIVER_URL'] + 'ads/atk_notice/')
-        if response.status_code == 200:
-            notice = json.loads(response.content.decode('utf-8'))['ads'][0]
-            if notice['ad_content'] == 'empty':
-                notice = None
+        try:
+            response = requests.get(
+                app.config['TILEDRIVER_URL'] + 'ads/atk_notice/')
+            if response.status_code == 200:
+                notice = json.loads(response.content.decode('utf-8'))['ads'][0]
+                if notice['ad_content'] == 'empty':
+                    notice = None
+        except RequestException:
+            return None
 
     return notice
 
@@ -82,7 +86,10 @@ def dismiss_notice():
 @home.route('/', methods=['GET'])
 @debug_only(app.config)
 def index():
-    response = requests.get(app.config['TILEDRIVER_URL'] + 'ads/atk_ad/')
+    try:
+        response = requests.get(app.config['TILEDRIVER_URL'] + 'ads/atk_ad/')
+    except RequestException:
+        response = None
 
     t_path = app.jinja_loader.searchpath[1]
     t_path = os.path.join(t_path, 'default_ads.json')
@@ -90,11 +97,14 @@ def index():
         default_obj = json.loads(ad_file.read())
         default_ads = default_obj['ads']
 
-    if response.status_code == 200:
-        ad_object = json.loads(response.content.decode('utf-8'))
-        ads = ad_object['ads']
-        if len(ads) < 3:
-            ads = ads + default_ads
+    if response:
+        if response.status_code == 200:
+            ad_object = json.loads(response.content.decode('utf-8'))
+            ads = ad_object['ads']
+            if len(ads) < 3:
+                ads = ads + default_ads
+        else:
+            ads = default_ads
     else:
         ads = default_ads
 
@@ -164,13 +174,23 @@ def main():
         iter_type = request.args.get('iter_type', None)
         iter_value = request.args.get('iter_value', None)
 
+    if status_key is None:
+        status_key = create_random_string(http_safe=True)
+
     if chain is None:
         return make_response('Missing chain parameter in request', 400)
 
-    try:
-        chain = json.loads(chain)
-    except ValueError:
-        return make_response('Chain parameter not properly formatted', 400)
+    if chain.lower() == 'from_global':
+        try:
+            chain = app.config['CHAIN_DATA'].pop(status_key)
+        except ValueError:
+            return make_response(
+                'Chain parameter not present in app configuration', 400)
+    else:
+        try:
+            chain = json.loads(chain)
+        except ValueError:
+            return make_response('Chain parameter not properly formatted', 400)
 
     if 'chain_name' not in chain:
         return make_response('Chain name not defined', 400)
@@ -178,9 +198,6 @@ def main():
     if 'algorithms' not in chain:
         return make_response('Algorithms not defined', 400)
 
-    if status_key is None:
-        status_key = create_random_string(http_safe=True)
-
     c_obj = AlgorithmChain(path, chain)
     if c_obj.chain_definition == {}:
         return make_response('Chain name not found', 404)
@@ -190,6 +207,7 @@ def main():
     if run_mode == 'single':
         response = c_obj.call_chain_algorithms()
         save_fname = status_key + '.json'
+
         if 'CHAIN_LEDGER_HISTORY_PATH' in app.config:
             save_path = os.path.join(
                 app.config['CHAIN_LEDGER_HISTORY_PATH'], save_fname)
@@ -197,6 +215,15 @@ def main():
             make_dir_if_not_exists(os.path.join(path, 'history'))
             save_path = os.path.join(path, 'history', save_fname)
         c_obj.chain_ledger.save_history_to_json(save_path, pretty=True)
+
+        if 'CHAIN_HISTORY' in app.config:
+            if app.config['CHAIN_HISTORY_LENGTH'] > 0:
+                ch = app.config['CHAIN_HISTORY']
+
+                if len(ch) > app.config['CHAIN_HISTORY_LENGTH']:
+                    ch.popitem(last=False)
+
+                ch[status_key] = c_obj.chain_ledger
     else:
         response = c_obj.call_batch(iter_param, iter_type, iter_value)
 
@@ -441,8 +468,9 @@ def create_algorithm(algorithm=None):
                                     temp_val = c_alg['parameters'][p_val]
                                     if 'source_algorithm' in temp_val:
                                         if (
-                                            temp_val['source_algorithm'] ==
-                                                algorithm
+                                            temp_val[
+                                                'source_algorithm'
+                                            ] == algorithm
                                         ):
                                             temp_val[
                                                 'source_algorithm'
@@ -878,7 +906,8 @@ def test_run(chain_name):
         for op in alg_definition['optional_parameters']:
             if get_source(a, op['name']) == 'user':
                 temp_field = set_field(op, '')
-                temp_name = a_name.replace(os.sep, '___') + '__' + op['name']
+                # temp_name = a_name.replace(os.sep, '___') + '__' + op['name']
+                temp_name = a_name.replace('/', '___') + '__' + op['name']
                 temp_name += '_' + str(a_iterator)
                 setattr(F, temp_name, temp_field)
                 op['field'] = eval('F().' + temp_name)
